# EPOW.R - Functions used for tests using exponential power distributions.


# LOG PROBABILITY DENSITY FUNCTION.  The power parameter is as specified by 
# "pow" for all distributons.  The shift parameter is eta times the value of
# "shift" specified below.  The scale parameter is the value of "scale"
# below raised to the power eta.

epow.lpr.evals <- 0

epow.lpr <- function (x, eta)
{
  epow.lpr.evals <<- epow.lpr.evals + 1

  - (abs(x-shift*eta)/scale^eta) ^ pow
}


# PLOT DENSITY FOR SEQUENCE OF DISTRIBUTIONS.

epow.dplt <- function (etas)
{
  low <- min (-3,shift-3*scale)
  high <- max (+3, shift+3*scale)
  plot(c(low,high),c(0,1.05),type="n",ylab="",yaxs="i",
    xlab=paste("q = ",pow,",  t = ",shift,",  s = ",scale,sep=""))
  g <- seq(low,high,length=1000)
  for (eta in etas)
  { lines(g,exp(epow.lpr(g,eta)))
  }
}


# GENERATE POINT FROM DISTRIBUTION WITH GIVEN ETA.  Done with rejection
# sampling from a uniform distribution that is wide enough to contain 
# virtually all the distribution.

epow.gen.calls <- 0

epow.gen <- function (eta)
{
  epow.gen.calls <<- epow.gen.calls + 1

  repeat
  { x <- runif (1, eta*shift-10*scale^eta, eta*shift+10*scale^eta)
    if (runif(1)<exp(epow.lpr(x,eta)))
    { return (x)
    }
  }
}


# TEST AIS ON THE EXPONENTIAL POWER DISTRIBUTIONS.  The .ind form uses
# independent sampling rather than Metropolis updates.

epow.ais.ind <- function (n.runs, n.steps, reverse=FALSE)
{ epow.ais (n.runs, n.steps, reverse,
    trans = function (x, lpr, eta, stepsize) epow.gen(eta)
  )
}

epow.ais <- function (n.runs, n.steps, reverse=FALSE, trans=met)
{
  res <- list()

  etas <- (1:n.steps)/n.steps

  for (r in 1:n.runs)
  { 
    if (reverse)
    { final <- epow.gen(1)
      ais.res <- 
        ais (epow.lpr, etas, trans, stepsize*scale^etas, final=final)
    }
    else
    { initial <- epow.gen(0)
      ais.res <- 
        ais (epow.lpr, etas, trans, stepsize*scale^etas, initial=initial)
    }

    res[[r]] <- ais.res
  }

  res
}


# TEST LIS ON THE EXPONENTIAL POWER DISTRIBUTIONS.  The .ind form uses
# independent sampling rather than Metropolis updates.

epow.lis.ind <- function (n.runs, n.steps, n.trans, reverse=FALSE, ...)
{ epow.lis (n.runs, n.steps, n.trans, reverse,
    trans = function (x, lpr, eta, stepsize) epow.gen(eta),
    ...
  )
}

epow.lis <- function (n.runs, n.steps, n.trans, reverse=FALSE, trans=met, ...)
{
  res <- list()

  etas <- (1:n.steps)/n.steps

  for (r in 1:n.runs)
  { 
    if (reverse)
    { final <- matrix (NA, nrow=n.trans+1, ncol=1)
      for (i in 1:nrow(final))
      { final[i,] <- epow.gen(1)
      }
      lis.res <- 
        lis (epow.lpr, etas, trans, stepsize*scale^c(0,etas), n.trans, 
             final=final, ...)
      lis.res$final <- final
    }
    else
    { initial <- matrix (NA, nrow=n.trans+1, ncol=1)
      for (i in 1:nrow(initial))
      { initial[i,] <- epow.gen(0)
      }
      lis.res <- 
        lis (epow.lpr, etas, trans, stepsize*scale^c(0,etas), n.trans, 
             initial=initial, ...)
      lis.res$initial <- initial
    }

    res[[r]] <- lis.res
  }

  res
}
