# MET.R - Metropolis transition function.


# METROPOLIS TRANSITION FUNCTION.  Performs one Metropolis transition from 
# state x, returning the new state (which may be the same as the old state).
# The proposal distribution used changes all components of x, with the change 
# to each component being independently drawn from the Gaussian distribution
# with mean zero and standard deviation given by the stepsize parameter.  The 
# change is accepted or rejected based on the log probability (or density) of 
# the current and proposed state, as evaluated by the lpr function, which is 
# passed a state along with the distribution parameter eta.

met.calls <- 0
met.rejections <- 0

met <- function (x, lpr, eta, stepsize)
{
  met.calls <<- met.calls + 1

  px <- x + rnorm(length(x),0,stepsize)

  d <- lpr(px,eta) - lpr(x,eta)

  if (!is.nan(d) && runif(1)<exp(d))
  { x <- px
  }
  else
  { met.rejections <<- met.rejections + 1
  }

  x
}
