# MVG.R - Functions used for tests using multivariate Gaussian distributions.


# LOG PROBABILITY FUNCTION.  The distributions are D-dimensional
# Gaussians with components that are independent, with means of zero
# and variances of exp(-K*(1+eta)).  The density functions used omit
# normalizing constants.

mvg.lpr <- function (x, eta)
{
  - sum(0.5*x^2/exp(-K*(1+eta)))
}


# GENERATE POINT FROM DISTRIBUTION WITH GIVEN ETA.  

mvg.gen.calls <- 0

mvg.gen <- function (eta)
{
  mvg.gen.calls <<- mvg.gen.calls + 1

  rnorm (D, 0, exp(-K*(1+eta)/2))
}


# AUTOREGRESSIVE UPDATE.

mvg.ar.calls <- 0

mvg.ar <- function (x, lpr, eta, alpha)
{ 
  mvg.ar.calls <<- mvg.ar.calls + 1

  alpha*x + rnorm(D,0,sqrt(1-alpha^2)*exp(-K*(1+eta)/2))
}


# TEST AIS ON THE MULTIVARIATE GAUSSIAN DISTRIBUTIONS.  

mvg.ais <- function (n.runs, n.steps, reverse=FALSE, trans=mvg.ar)
{
  res <- list()

  etas <- (1:n.steps)/n.steps

  for (r in 1:n.runs)
  { 
    if (reverse)
    { final <- mvg.gen(1)
      ais.res <- ais (mvg.lpr, etas, trans, alpha, final=final)
    }
    else
    { initial <- mvg.gen(0)
      ais.res <- ais (mvg.lpr, etas, trans, alpha, initial=initial)
    }

    res[[r]] <- ais.res
  }

  res
}


# TEST LIS ON THE MULTIVARIATE GAUSSIAN DISTRIBUTIONS. 

mvg.lis <- function (n.runs, n.steps, n.trans, reverse=FALSE, trans=mvg.ar,...)
{
  res <- list()

  etas <- (1:n.steps)/n.steps

  for (r in 1:n.runs)
  { 
    if (reverse)
    { final <- matrix (NA, nrow=n.trans+1, ncol=D)
      for (i in 1:nrow(final))
      { final[i,] <- mvg.gen(1)
      }
      lis.res <- lis (mvg.lpr, etas, trans, alpha, n.trans, final=final, ...)
      lis.res$final <- final
    }
    else
    { initial <- matrix (NA, nrow=n.trans+1, ncol=D)
      for (i in 1:nrow(initial))
      { initial[i,] <- mvg.gen(0)
      }
      lis.res <- lis (mvg.lpr, etas, trans, alpha, n.trans, initial=initial,...)
      lis.res$initial <- initial
    }

    res[[r]] <- lis.res
  }

  res
}
