# TEST-PLT.R - Plot results produced by the "test" function.


# PLOT RESULTS ON MSE ESTIMATING LOG OF R.  The first argument is the
# result of "test".  The "by" argument gives the intervals at which to
# draw horizontal lines on the bar plot.  The "max" argument is used
# to limit the height of bars. 

test.plt.mse.log <- function (d, by, max=Inf, ylim=NULL, ylab="")
{
  use   <- c( 1,2,4, 5,6,10,  7,8,11)
  space <- c( 1,0,0, 1,0,0,   1,0,0 )

  mse <- d[use,]$mse.log
  se <- d[use,]$mse.log.se

  mseo <- pmin(mse,max)
  seo <- se
  seo[mse>max] <- 0
  
  if (is.null(ylim))
  { ylim <- max(mseo*1.1)
  }

  barplot (rbind(mseo-2*seo,4*seo), space=space, ylim=c(0,ylim), ylab=ylab)

  for (i in 1:length(mse))
  { if (mse[i]>max)
    { text (i+sum(space[1:i])-0.5, max*1.03, as.character(round(mse[i],2)))
    }
  }

  abline(h=0)
  abline(h=seq(by,ylim,by=by),lty=3)

  mtext ("AIS", 3, 0.3, at=2.5, cex=1.3)

  mtext ("for", 1, 0.2, at=1.5)
  mtext ("rev", 1, 0.2, at=2.5)
  mtext ("bri", 1, 0.2, at=3.5)

  mtext ("LIS-geometric", 3, 0.3, at=6.5, cex=1.3)

  mtext ("for", 1, 0.2, at=5.5)
  mtext ("rev", 1, 0.2, at=6.5)
  mtext ("bri", 1, 0.2, at=7.5)

  mtext ("LIS-optimal", 3, 0.3, at=10.5, cex=1.3)

  mtext ("for", 1, 0.2, at=9.5)
  mtext ("rev", 1, 0.2, at=10.5)
  mtext ("bri", 1, 0.2, at=11.5)
}


# PRODUCE A POSTSCRIPT PLOT OF A PAIR OF BAR GRAPHS.

test.plt.pair <- function (file, d1, d2, by1, by2, max1=Inf, max2=Inf, 
                           ylim=NULL, titles=rep("",2))
{
  postscript(file,horizontal=F,pointsize=7,width=6.5,height=3.9)
  par(mfrow=c(1,2),mar=c(2,4,4,1)+0.1)
  test.plt.mse.log(d1,by1,max1,ylim=ylim,ylab="Mean Squared Error")
  mtext (titles[1], 3, 2.6, at=6.5, cex=1.8)
  test.plt.mse.log(d2,by2,max2,ylim=ylim)
  mtext (titles[2], 3, 2.6, at=6.5, cex=1.8)
  dev.off()
}


# PRODUCE A POSTSCRIPT PLOT OF FOUR BAR GRAPHS.

test.plt.four <- function (file, d1, d2, d3, d4, by, max=Inf, ylim=NULL, 
                           titles=rep("",4))
{
  postscript(file,horizontal=F,pointsize=7,width=6.5,height=7)
  par(mfrow=c(2,2),mar=c(2,4,6,1)+0.1)
  test.plt.mse.log(d1,by,max,ylim=ylim,ylab="Mean Squared Error")
  mtext (titles[1], 3, 2.6, at=6.5, cex=1.8)
  test.plt.mse.log(d2,by,max,ylim=ylim)
  mtext (titles[2], 3, 2.6, at=6.5, cex=1.8)
  test.plt.mse.log(d3,by,max,ylim=ylim,ylab="Mean Squared Error")
  mtext (titles[3], 3, 2.6, at=6.5, cex=1.8)
  test.plt.mse.log(d4,by,max,ylim=ylim)
  mtext (titles[4], 3, 2.6, at=6.5, cex=1.8)
  dev.off()
}
